#include "cPlot.h"
//#include "cPlotUtility.h"
#include "cWindow.h"
#include "mathlib.h"
#include <strstream>
#include <string>

#if defined(_WIN32)
	#include "cFont.h"
#endif

using namespace std;

cPlot::cPlot()
{
	//typeRender = nSURFACE;
	bPoint = false;
	bWireframe = false;
	bSurface = true;

	bGraph = true;
	bBox = false;
	bAxis = true;
	bArrow = false;
	bScale = true;

	//bZooming = (mode == nZoom);
}

void cPlot::Render()
{
    glPushMatrix();
	{
	    //set up projection
		glMatrixMode(GL_PROJECTION);
	    glLoadIdentity();

		gluPerspective(	perspective[0].value, perspective[1].value, 
						perspective[2].value, perspective[3].value);

		
		const cFloat PI=3.1415926;
		
		cFloat Long = clongitude.value;
		cFloat Lat = clattitude.value;
		cFloat R = cradius.value;

		cFloat D1 = Long*2.0*PI/360.0;
		cFloat D2 = Lat*2.0*PI/360.0;
		
		lookat[0].value = R*cos(D2)*cos(D1);
		lookat[1].value = R*sin(D2);
		lookat[2].value = R*cos(D2)*sin(D1);

		//shift from center
		lookat[0].value += lookat[3].value;
		lookat[1].value += lookat[4].value;
		lookat[2].value += lookat[5].value;
		
		gluLookAt(	lookat[0].value, lookat[1].value, lookat[2].value,
					lookat[3].value, lookat[4].value, lookat[5].value,
				    lookat[6].value, lookat[7].value, lookat[8].value);

		glGetDoublev(GL_PROJECTION_MATRIX, matProjection);	//get the matrix for field of view's use

		//set up model matrix
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

	    glPushMatrix();
		{

			glRotatef(rotate[0].value, 1.0, 0.0, 0.0);
			glRotatef(rotate[1].value, 0.0, 1.0, 0.0);
			glRotatef(rotate[2].value, 0.0, 0.0, 1.0);

			glScalef(scale[0].value, scale[1].value, scale[2].value);
			glScalef(zoom.value, zoom.value, zoom.value);

		    glGetDoublev(GL_MODELVIEW_MATRIX, matModel);

			//render the dataset
			if (bGraph)
			{
				RenderGraph();
			}

			//render box
			if (bBox)
			{
				glColor3f(1.0f, 1.0f, 1.0f);
				glEnable(GL_DEPTH_TEST);
				RenderBox();
			}

			if (bAxis)
			{

				//dotted line for the part under surface
				glDisable(GL_DEPTH_TEST);
				glEnable(GL_LINE_STIPPLE);

				int stipple = 0x00ff;
				glLineStipple(1, (GLushort)stipple);

				/* this turn out to be a diaster
				glEnable(GL_POLYGON_STIPPLE);

				GLubyte pstipple[4*32];
				GLubyte base = 0x6666;
				for(int i=0; i<4; i++)
				{
					GLubyte style;
					if (i%4 <= 1) style = base;
					else style = ~base;

					for(int j=0; j<8; j++)
					{
						pstipple[i+j*4] = stipple;
					}
				}
				glPolygonStipple(&pstipple[0]);
				*/

				RenderAxis();

				//solid line for visible part
				glDisable(GL_LINE_STIPPLE);
				glDisable(GL_POLYGON_STIPPLE);
				glEnable(GL_DEPTH_TEST);

				RenderAxis();
			}

			//Render the side
			RenderSide();

			//Render title
			RenderTitle();

		} glPopMatrix();
	} glPopMatrix();
}

//get the starting
cFloat GetStart(int Axis)
{
	int i = Axis;
	cFloat start = min[i].value;
	start -= fmod(min[i].value, tinc[i].value);
	if (start < min[i].value) start += tinc[i].value;

	return start;
}

void cPlot::RenderSide()
{
	const int X = 0;
	const int Y = 1;
	const int Z = 2;

	//for each of the 6 side
	for(int s=0; s<6; s++)
	{
		//if turn off, don't draw
		if (sOn[s].IsOff()) continue;
		if (sGrid[s].IsOff() && sFill[s].IsOff()) continue;

		//draw the surface, then translate appropriately
		cFloat amin1, amin2;	//actual min
		cFloat min1, min2;
		cFloat max1, max2;
		cFloat inc1, inc2;
		bool fill = sFill[s].IsOn();
		bool grid = sGrid[s].IsOn();

		switch (s)
		{
		case sFRONT:
		case sBACK:
			amin1 = min[X].value; min1 = GetStart(X); max1 = max[X].value; inc1 = tinc[X].value;
			amin2 = min[Y].value; min2 = GetStart(Y); max2 = max[Y].value; inc2 = tinc[Y].value;
			break;
		case sLEFT:
		case sRIGHT:
			amin1 = min[Y].value; min1 = GetStart(Y); max1 = max[Y].value; inc1 = tinc[Y].value;
			amin2 = min[Z].value; min2 = GetStart(Z); max2 = max[Z].value; inc2 = tinc[Z].value;
			break;
		case sTOP:
		case sBOT:
			amin1 = min[Z].value; min1 = GetStart(Z); max1 = max[Z].value; inc1 = tinc[Z].value;
			amin2 = min[X].value; min2 = GetStart(X); max2 = max[X].value; inc2 = tinc[X].value;
			break;
		}

		glPushMatrix();
		{
			switch (s)
			{
			case sFRONT:	glTranslatef(0.0, 0.0, min[Z].value); break;
			case sBACK:		glTranslatef(0.0, 0.0, max[Z].value); break;
			case sLEFT:		glTranslatef(min[X].value, 0.0, 0.0); break;
			case sRIGHT:	glTranslatef(max[X].value, 0.0, 0.0); break;
			case sTOP:		glTranslatef(0.0, min[Y].value, 0.0); break;
			case sBOT:		glTranslatef(0.0, max[Y].value, 0.0); break;
			};

			if (fill)
			{
				glColor3f(0.5f, 0.5f, 0.5f);
				glBegin(GL_QUADS);
				{
					switch (s)
					{
					case sFRONT:
					case sBACK:
						glVertex3f(amin1, max2, 0.0f);
						glVertex3f(max1, max2, 0.0f);
						glVertex3f(max1, amin2, 0.0f);
						glVertex3f(amin1, amin2, 0.0f);
						break;
					case sLEFT:		
					case sRIGHT:
						glVertex3f(0.0f, amin1, max2);
						glVertex3f(0.0f, max1, max2);
						glVertex3f(0.0f, max1, amin2);
						glVertex3f(0.0f, amin1, amin2);
						break;
					case sTOP:
					case sBOT:
						glVertex3f(max2, 0.0f, amin1);
						glVertex3f(max2, 0.0f, max1);
						glVertex3f(amin2, 0.0f, max1);
						glVertex3f(amin2, 0.0f, amin1);
						break;
					};
				}
				glEnd();
			}

			glColor3f(1.0, 1.0, 1.0);

			//border line
			glBegin(GL_LINE_LOOP);
			{
				switch (s)
				{
				case sFRONT:
				case sBACK:
					glVertex3f(amin1, max2, 0.0f);
					glVertex3f(max1, max2, 0.0f);
					glVertex3f(max1, amin2, 0.0f);
					glVertex3f(amin1, amin2, 0.0f);
					break;
				case sLEFT:		
				case sRIGHT:
					glVertex3f(0.0f, amin1, max2);
					glVertex3f(0.0f, max1, max2);
					glVertex3f(0.0f, max1, amin2);
					glVertex3f(0.0f, amin1, amin2);
					break;
				case sTOP:
				case sBOT:
					glVertex3f(max2, 0.0f, amin1);
					glVertex3f(max2, 0.0f, max1);
					glVertex3f(amin2, 0.0f, max1);
					glVertex3f(amin2, 0.0f, amin1);
					break;
				};
			}
			glEnd();

			//TODO since fill and grid doesn't work well together, when both on, only use fill
			if (grid && !fill)
			{
				//actual grid

				glEnable(GL_POLYGON_OFFSET_LINE);
				glPolygonOffset(1.0, 1.0);

				glBegin(GL_LINES);

				for(cFloat x=min1; x<=max1; x+=inc1)
				{
					switch (s)
					{
					case sFRONT:
					case sBACK:
						glVertex3f(x, amin2, 0.0f);
						glVertex3f(x, max2, 0.0f);
						break;
					case sLEFT:		
					case sRIGHT:
						glVertex3f(0.0f, x, amin2);
						glVertex3f(0.0f, x, max2);
						break;
					case sTOP:
					case sBOT:
						glVertex3f(amin2, 0.0f, x);
						glVertex3f(max2, 0.0f, x);
						break;
					};
				}

				for(cFloat y=min2; y<=max2; y+=inc2)
				{
					switch (s)
					{
					case sFRONT:
					case sBACK:
						glVertex3f(amin1, y, 0.0f);
						glVertex3f(max1, y, 0.0f);
						break;
					case sLEFT:		
					case sRIGHT:
						glVertex3f(0.0f, amin1, y);
						glVertex3f(0.0f, max1, y);
						break;
					case sTOP:
					case sBOT:
						glVertex3f(y, 0.0f, amin1);
						glVertex3f(y, 0.0f, max1);
						break;
					};
				}

				glEnd();

				glDisable(GL_POLYGON_OFFSET_LINE);
			}

		} glPopMatrix();
	}


}

void cPlot::RenderTitle()
{
	const int X = 0;
	const int Y = 1;
	const int Z = 2;

	//get the title
	string title = cTitle.str;
	int length = title.length();
	int mid = length / 2;

	int cut = -1;
	/* try to cut at blank
	//figure out the cutoff
	for(int d=0; d<mid/2; d++)
	{
		if (title[mid-d] == ' ')
		{
			cut = mid-d;
			break;
		}
		
		if (title[mid+d] == ' ')
		{
			cut = mid+d;
			break;
		}
	}
	*/
	
	//if there is no way to cut, just cut in the middle
	if (cut == -1) cut = mid;

	//cut appropriately
	string t1 = title.substr(0, cut);
	string t2 = title.substr(cut, title.length()-cut);

	//render it
	int corner = cCorner.value / 90.0;

	glPushMatrix();
	{
		switch (corner)
		{
		case 0:	glTranslatef(max[X].value, max[Y].value, max[Z].value); break;
		case 1:	glTranslatef(min[X].value, max[Y].value, max[Z].value); break;
		case 2:	glTranslatef(min[X].value, min[Y].value, max[Z].value); break;
		case 3:	glTranslatef(max[X].value, min[Y].value, max[Z].value); break;
		}

		//rotate to the right corner
		glRotatef(90.0*corner, 0.0, 0.0, 1.0);
		glScalef(0.5, 0.5, 0.5);

		glColor3f(1.0, 1.0, 0.2);
		glRotatef(90.0, 1.0, 0.0, 0.0);
		glPrint(jRIGHT, t1.c_str());

		glRotatef(-90.0, 0.0, 1.0, 0.0);
		glPrint(jLEFT, t2.c_str());

	} glPopMatrix();

}

void cPlot::RenderBox()
{
	const cFloat
		maxX = max[0].value,
		maxY = max[1].value,
		maxZ = max[2].value;

	const cFloat
		minX = min[0].value,
		minY = min[1].value,
		minZ = min[2].value;

	//Y-Z plane
	glBegin(GL_LINE_LOOP);
	glVertex3f(minX, minY, minZ);
	glVertex3f(minX, minY, maxZ);
	glVertex3f(minX, maxY, maxZ);
	glVertex3f(minX, maxY, minZ);
	glEnd();

	glBegin(GL_LINE_LOOP);
	glVertex3f(maxX, minY, minZ);
	glVertex3f(maxX, minY, maxZ);
	glVertex3f(maxX, maxY, maxZ);
	glVertex3f(maxX, maxY, minZ);
	glEnd();

	//4 edge connecting maxX and minX
	glBegin(GL_LINES);
	glVertex3f(maxX, minY, minZ);
	glVertex3f(minX, minY, minZ);

	glVertex3f(maxX, minY, maxZ);
	glVertex3f(minX, minY, maxZ);
	
	glVertex3f(maxX, maxY, maxZ);
	glVertex3f(minX, maxY, maxZ);
	
	glVertex3f(maxX, maxY, minZ);
	glVertex3f(minX, maxY, minZ);
	glEnd();
}

string ToStr(cFloat f)
{
	strstream str;
	str << f;
	string res;
	str >> res;
	return res;
}

void DrawNum(cFloat n, cFloat x, cFloat y, cFloat z)
{
	glColor3ub(255, 255, 0);
	glRasterPos3f(x, y, z);
	
	//glPrint("Hello");

	string str = ToStr(n).c_str();
	const char* s = str.c_str();
	for (s; *s; s++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, *s);
}

void DrawStr(string str, cFloat x, cFloat y, cFloat z)
{
	//glColor3ub(255, 255, 0);
	glRasterPos3f(x, y, z);
	
	const char* s = str.c_str();

	
	for (s; *s; s++)
	{
	//	glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, *s);
		glutStrokeCharacter(GLUT_STROKE_ROMAN, *s);

	}
	
}

void cPlot::RenderAxis()
{
	const int X = 0;
	const int Y = 1;
	const int Z = 2;

	//get all the settings

	//decide which corner to put axis
	int corner[3];
	//TODO, add this into setting menu
	corner[X] = 0;
	corner[Y] = 0;
	corner[Z] = 2;

	glMatrixMode(GL_MODELVIEW);
	
	glPushMatrix();
	{
		//create the direction axis
		cFloat actcenter[3]; //actual center
		cFloat invcenter[3]; //inversed center

		for(int c=0; c<3; c++)
		{
			actcenter[c] = (max[c].value - min[c].value) * loc[c].value + min[c].value;
			invcenter[c] = (min[c].value - max[c].value) * loc[c].value + max[c].value;
		}

		if (bScale)
		{
			//font
			/*
			#if defined(_WIN32)
				glEnable(GL_DEPTH_TEST);
			#endif
			*/

			//draw tick
			//for x, y, z

			cFloat center[3];

			for(int i=0; i<3; i++)
			{
				int nLast = (i+2) % 3;
				int nThis = (i+0) % 3;
				int nNext = (i+1) % 3;

				//get the each corner appropriated
				switch (corner[i])
				{
				case 0:
					center[nLast] = actcenter[nLast];
					center[nThis] = actcenter[nThis];
					center[nNext] = actcenter[nNext];
					break;
				case 1:
					center[nLast] = actcenter[nLast];
					center[nThis] = actcenter[nThis];
					center[nNext] = invcenter[nNext];
					break;
				case 2:
					center[nLast] = invcenter[nLast];
					center[nThis] = actcenter[nThis];
					center[nNext] = actcenter[nNext];
					break;
				case 3:
					center[nLast] = invcenter[nLast];
					center[nThis] = actcenter[nThis];
					center[nNext] = invcenter[nNext];
					break;
				}
				
				//draw the axis
				glColor3ub(cAXISR, cAXISG, cAXISB);
				glBegin(GL_LINES);
				switch (i)
				{
				case X:
					glVertex3f(min[0].value, center[1], center[2]);
					glVertex3f(max[0].value, center[1], center[2]);
					break;
				case Y:
					glVertex3f(center[0], min[1].value, center[2]);
					glVertex3f(center[0], max[1].value, center[2]);
					break;
				case Z:
					glVertex3f(center[0], center[1], min[2].value);
					glVertex3f(center[0], center[1], max[2].value);
					break;
				}
				glEnd();

				//write out the caption for each axis
				glPushMatrix();
				{
					const cFloat DIST = 1.0;
					switch (i)
					{
					case X:
						glTranslatef((min[0].value + max[0].value) / 2.0, center[1], center[2]);
						glTranslatef(0.0, -DIST, 0.0);
						break;
					case Y:
						glTranslatef(center[0], (min[1].value + max[1].value) / 2.0, center[2]);
						glTranslatef(-DIST, 0.0, 0.0);
						glRotatef(-90.0, 0.0, 0.0, 1.0);
						break;
					case Z:
						glTranslatef(center[0], center[1], (min[2].value + max[2].value) / 2.0);
						glTranslatef(-DIST, 0.0, 0.0);
						glRotatef(90.0, 1.0, 0.0, 0.0);
						glRotatef(90.0, 0.0, 0.0, 1.0);
						break;
					}

					cFloat scale = tinc[i].value * fSize[i].value;
					glScalef(scale, scale, scale);
					glPrint(jCENTER, "%s", cAxis[i].str.c_str());

				}glPopMatrix();


				//make sure the tick will mark origin
				cFloat start = min[i].value;
				start -= fmod(min[i].value, tinc[i].value);
				if (start < min[i].value) start += tinc[i].value;

				for(cFloat j=start; j<=max[i].value; j+=tinc[i].value)
				{
					//don't draw if it's at the intersection of axis
					if (j == center[i]) continue;

					if (i == X && fX.IsOn())
					{
						glColor3ub(cAXISR, cAXISG, cAXISB);
						glBegin(GL_LINES);
						glVertex3f(j, center[1], center[2]);
						glVertex3f(j, center[1]+tick[i].value, center[2]);
						glEnd();
					
						//labels
						glPushMatrix();
						{
							//font
							#if defined(_WIN32)
								if (f3D.IsOn())
								{
									cFloat scale = tinc[i].value * fSize[i].value;

									glColor3ub(fXColor[0].value, fXColor[1].value, fXColor[2].value);
									glTranslatef(j, center[1], center[2]);
									glScalef(scale, scale, scale);
									glRotatef(-90.0, 0.0, 0.0, 1.0);
									glPrint(jLEFT, "%+.2f", j);
								}
								else
								{
									DrawNum(j, j, center[1], center[2]);
								}
							#else
								DrawNum(j, j, center[1], center[2]);
							#endif

						} glPopMatrix();

					}

					if (i == Y && fY.IsOn())
					{
						glColor3ub(cAXISR, cAXISG, cAXISB);
						glBegin(GL_LINES);
						glVertex3f(center[0], j, center[2]);
						//glVertex3f(center[0], j, center[2]+tick[i].value);
						glVertex3f(center[0]+tick[i].value, j, center[2]);
						glEnd();

						//labels
						glPushMatrix();
						{
							//font
							#if defined(_WIN32)
								if (f3D.IsOn())
								{
									cFloat scale = tinc[i].value * fSize[i].value;

									glColor3ub(fYColor[0].value, fYColor[1].value, fYColor[2].value);
									glTranslatef(center[0], j, center[2]);
									glScalef(scale, scale, scale);
									//glRotatef(0.0, 0.0, 1.0, 0.0);
									glPrint(jRIGHT, "%+.2f ", j);
								}
								else
								{
									DrawNum(j, center[0], j, center[2]);
								}
							#else
								DrawNum(j, center[0], j, center[2]);
							#endif

						} glPopMatrix();
					}

					if (i == Z && fZ.IsOn())
					{
						glColor3ub(cAXISR, cAXISG, cAXISB);
						glBegin(GL_LINES);
						glVertex3f(center[0], center[1], j);
						glVertex3f(center[0]+tick[i].value, center[1], j);
						glEnd();

						glPushMatrix();
						{
							//font
							#if defined(_WIN32)
								if (f3D.IsOn())
								{
									cFloat scale = tinc[i].value * fSize[i].value;

									glColor3ub(fZColor[0].value, fZColor[1].value, fZColor[2].value);
									glTranslatef(center[0], center[1], j);
									glScalef(scale, scale, scale);
									glRotatef(90.0, 1.0, 0.0, 0.0);
									glPrint(jRIGHT, "%+.2f ", j);
								}
								else
								{
									DrawNum(j, center[0], center[1], j);
								}
							#else
									DrawNum(j, center[0], center[1], j);
							#endif

						} glPopMatrix();
					}
				}
			}
		}

		//draw the caption for each axis


		if (bArrow)
		{
			//draw the arrows, with line to arrows
			glTranslatef(actcenter[0], actcenter[1], actcenter[2]); 
			glScalef(max[0].value - actcenter[0], max[1].value - actcenter[1], max[2].value-actcenter[2]);
			glColor3ub(cAXISR, cAXISG, cAXISB);

			glBegin(GL_LINE_STRIP);
			glVertex3f(0.0, 0.0, 0.0);
			glVertex3f(1.0, 0.0, 0.0);
			glVertex3f(0.75, 0.25, 0.0);
			glVertex3f(0.75, -0.25, 0.0);
			glVertex3f(1.0, 0.0, 0.0);
			glVertex3f(0.75, 0.0, 0.25);
			glVertex3f(0.75, 0.0, -0.25);
			glVertex3f(1.0, 0.0, 0.0);
			glEnd();
			glBegin(GL_LINE_STRIP);
			glVertex3f(0.0, 0.0, 0.0);
			glVertex3f(0.0, 1.0, 0.0);
			glVertex3f(0.0, 0.75, 0.25);
			glVertex3f(0.0, 0.75, -0.25);
			glVertex3f(0.0, 1.0, 0.0);
			glVertex3f(0.25, 0.75, 0.0);
			glVertex3f(-0.25, 0.75, 0.0);
			glVertex3f(0.0, 1.0, 0.0);
			glEnd();
			glBegin(GL_LINE_STRIP);
			glVertex3f(0.0, 0.0, 0.0);
			glVertex3f(0.0, 0.0, 1.0);
			glVertex3f(0.25, 0.0, 0.75);
			glVertex3f(-0.25, 0.0, 0.75);
			glVertex3f(0.0, 0.0, 1.0);
			glVertex3f(0.0, 0.25, 0.75);
			glVertex3f(0.0, -0.25, 0.75);
			glVertex3f(0.0, 0.0, 1.0);
			glEnd();
		
			//draw the font
			glColor3ub(255, 255, 0);
			if (f3D.IsOn())
			{
				glPushMatrix();
				glTranslatef(1.1, 0.0, 0.0);
				glScalef(0.2, 0.2, 0.2);
				glPrint(jLEFT, "x");
				glPopMatrix();
				
				glPushMatrix();
				glTranslatef(0.0, 1.1, 0.0);
				glScalef(0.2, 0.2, 0.2);
				glPrint(jLEFT, "y");
				glPopMatrix();

				glPushMatrix();
				glTranslatef(0.0, 0.0, 1.1);
				glScalef(0.2, 0.2, 0.2);
				glRotatef(90.0, 1.0, 0.0, 0.0);
				glPrint(jLEFT, "z");
				glPopMatrix();
			}
			else
			{
				glRasterPos3f(1.1, 0.0, 0.0);
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, 'x');
				glRasterPos3f(0.0, 1.1, 0.0);
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, 'y');
				glRasterPos3f(0.0, 0.0, 1.1);
				glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, 'z');
			}

			glColor3ub(255, 150, 150);
			string centerLoc = "(" + ToStr(actcenter[0]) + ", " + ToStr(actcenter[1]) + ", " + ToStr(actcenter[2]) + ")";
			//DrawStr(centerLoc, center[0], center[1], center[2]);
			//TODO for some reason, this doesn't seem to display naymore
			DrawStr(centerLoc, -0.04, -0.04, -0.04);
		}		
	} glPopMatrix();
}

void cPlot::RenderGraph()
{
	const cFloat
		maxX = (max[0].value + center[0].value) / zoom.value,
		maxY = (max[1].value + center[1].value) / zoom.value,
		maxZ = (max[2].value + center[2].value) / zoom.value;

	const cFloat
		minX = (min[0].value + center[0].value) / zoom.value,
		minY = (min[1].value + center[1].value) / zoom.value,
		minZ = (min[2].value + center[2].value) / zoom.value;

	cFloat
		incX = inc[0].value * zoom.value,
		incY = inc[1].value * zoom.value,
		incZ = inc[2].value * zoom.value;	//not used

	const cFloat
		xRange = maxX - minX,
		yRange = maxY - minY;

	int
		xStep = (int(xRange / incX) + 1.0),
		yStep = (int(yRange / incY) + 1.0);

	if (xStep >= MAXSTEP) xStep = MAXSTEP;
	if (yStep >= MAXSTEP) yStep = MAXSTEP;

	//use the closest divider
	incX = xRange / cFloat(xStep);
	incY = yRange / cFloat(yStep);

	cDataSet* theData = data[int(plotIndex.value+0.5)];

	// Clear the background
	glClear(GL_COLOR_BUFFER_BIT);

	const cFloat alpha = 0.5;

	if (bSurface)
	{
		glEnable(GL_DEPTH_TEST);
		glBegin(GL_QUADS);

		for(int j=0; j<=yStep-1; j++)
		{
			cFloat y = minY + cFloat(j)*incY;
			for(int i=0; i<=xStep-1; i++)
			{
				cFloat x = minX + cFloat(i)*incX;

				cFloat z1 = theData->GetZ(x, y);
				cFloat z2 = theData->GetZ(x+incX, y);
				cFloat z3 = theData->GetZ(x+incX, y+incY);
				cFloat z4 = theData->GetZ(x, y+incY);

				cColor c1 = theData->GetColor(z1);
				cColor c2 = theData->GetColor(z2);
				cColor c3 = theData->GetColor(z3);
				cColor c4 = theData->GetColor(z4);

				glColor4f(c1[0], c1[1], c1[2], alpha);
				glVertex3f(x, y, z1);
				
				glColor4f(c2[0], c2[1], c2[2], alpha);
				glVertex3f(x+incX, y, z2);

				glColor4f(c3[0], c3[1], c3[2], alpha);
				glVertex3f(x+incX, y+incY, z3);

				glColor4f(c4[0], c4[1], c4[2], alpha);
				glVertex3f(x, y+incY, z4);
			}
		}

		glEnd();
	}

	//if (typeRender == nPOINT)
	if (bPoint)
	{
		glDisable(GL_DEPTH_TEST);
		glBegin(GL_POINTS);

		for(int j=0; j<=yStep; j++)
		{
			cFloat y = minY + cFloat(j)*incY;
			for(int i=0; i<=xStep; i++)
			{
				cFloat x = minX + cFloat(i)*incX;

				cFloat z = theData->GetZ(x, y);
				cColor color = theData->GetColor(z);

				if (bSurface) color = cColor(0.0, 1.0, 0.0);

				glColor3f(color[0], color[1], color[2]);
				glVertex3f(x, y, z);
				
			}
		}
		glEnd();
	}

	//wireframe
	//if (typeRender == nWIREFRAME)
	if (bWireframe)
	{
		for(int j=0; j<=yStep; j++)
		{
			cFloat y = minY + cFloat(j)*incY;

			glBegin(GL_LINE_STRIP);
			for(int i=0; i<=xStep; i++)
			{
				cFloat x = minX + cFloat(i)*incX;

				cFloat z = theData->GetZ(x, y);
				cColor color = theData->GetColor(z);

				//to make the wireframe visible
				if (bSurface)
				{
					color = cColor(0.0, 1.0, 0.0);
					if (z > 0)	z = z + 0.01*z;
					else		z = z - 0.01*z;
				}

				glColor3f(color[0], color[1], color[2]);
				glVertex3f(x, y, z);
				
			}
			glEnd();
		}

		for(int i=0; i<=xStep; i++)
		{
			cFloat x = minX + cFloat(i)*incX;

			glBegin(GL_LINE_STRIP);
			for(int j=0; j<=yStep; j++)
			{
				cFloat y = minY + cFloat(j)*incY;

				cFloat z = theData->GetZ(x, y);
				cColor color = theData->GetColor(z);

				//to make the wireframe visible
				if (bSurface)
				{
					color = cColor(0.0, 1.0, 0.0);
					if (z > 0)	z = z + 0.01*z;
					else		z = z - 0.01*z;
				}

				glColor3f(color[0], color[1], color[2]);
				glVertex3f(x, y, z);
				
			}
			glEnd();
		}
	}

	glFinish();
	
}