#include "cWindowMain.h"
#include "cWindowConfig.h"
#include "cWindow.h"
#include "cInfo.h"
#include "cFont.h"
#include <string>
#include <strstream>
#include <iostream>

using namespace std;

void initMain()
{
	glutDisplayFunc(displayMain);
	glutReshapeFunc(reshapeMain);
	glutKeyboardFunc(keyboardMain);
//	glutIdleFunc(idleMain);

	glutCreateMenu(menuMain);
	glutAddMenuEntry("Save", 's');
	glutAddMenuEntry("Load", 'l');
    glutAttachMenu(GLUT_RIGHT_BUTTON);
}


//THIS SHOULD NOT BE NEEDE
void idleMain()
{
	cout << "idle called";
	RedisplayAll();
	//glutPostRedisplay();
}


void displayMain()
{
	
    glClearColor(0.8f, 0.8f, 0.8f, 0.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	int viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);

	const int gapTITLE = 5;
	int width = viewport[2];
	int height = viewport[3];

	//compute ratio
	GLdouble rWidth = GLdouble(width) / (mWIDTH + 3*GAP);
	GLdouble rHeight = GLdouble(height) / (mHEIGHT + 2*GAP);

	//display action
    setfont("helvetica", 18);
	glColor3ub(0, 0, 0);
	drawstr(pLEFT * rWidth, pTOP * rHeight - gapTITLE, "Graph:");
	drawstr(wLEFT * rWidth, wTOP * rHeight - gapTITLE, "Field of View:");

	string setting = "Settings: ";
	if (mode == nColor) setting += "Color Mapping";
	if (mode == nView) setting += "3D View";
	if (mode == nAxis) setting += "Axis Range";
	if (mode == nZoom) setting += "Zoom In/Out";
	if (mode == nCamera) setting += "Camera Position";
	if (mode == nFont) setting += "Axis Font";
	if (mode == nCaption) setting += "Caption";

	drawstr(cLEFT * rWidth, cTOP * rHeight - gapTITLE, setting);

	/*
	//font
	BuildFont();
	glBegin(GL_LINES)
	glVertex3f(0, 0, 0);
	glVertex3f(100, 100, 100);
	glEnd();
	glPrint("hello");
*/


    glutSwapBuffers();
};

void reshapeMain(int width, int height)
{

    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0, width, height, 0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glClearColor(0.0, 0.0, 0.0, 0.0);
	
	//for debugging
	//width = 20;
	//height = 20;


	//compute ratio
	GLdouble rWidth = GLdouble(width) / (mWIDTH + 3*GAP);
	GLdouble rHeight = GLdouble(height) / (mHEIGHT + 2*GAP);

    glutSetWindow(winPlot);
    glutPositionWindow(pLEFT * rWidth, pTOP * rHeight);
    glutReshapeWindow(pWIDTH * rWidth, pHEIGHT * rHeight);

    glutSetWindow(winWorld);
    glutPositionWindow(wLEFT * rWidth, wTOP * rHeight);
    glutReshapeWindow(wWIDTH * rWidth, wHEIGHT * rHeight);

    glutSetWindow(winConfig);
    glutPositionWindow(cLEFT * rWidth, cTOP * rHeight);
    glutReshapeWindow(cWIDTH * rWidth, cHEIGHT * rHeight);

}

void menuMain(int value)
{
	string index;
	
	//convert plotIndex to string
	strstream str;
	str << int(plotIndex.value+0.5);
	str >> index;

	switch (value)
	{
	case 's':	Save("plot" + index + ".plt"); break;
	case 'l':	Load("plot" + index + ".plt"); break;
	}

	//RedisplayAll();

	return;
}

void keyboardMain(unsigned char ch, int x, int y)
{
	/*
	switch (ch)
	{
	case '0': 
	case '1':
	case '2':
	case '3': plotIndex.value = int(ch - unsigned char('0')); break;
	}

	RedisplayAll();
	*/
}


bool SaveCells(ofstream& out, cell* c, int num)
{
	for(int i=0; i<num; i++)
	{
		out << (*c);
		c++;
	}

	return true;
}

bool LoadCells(ifstream& in, cell* c, int num)
{
	for(int i=0; i<num; i++)
	{
		in >> (*c);
		c++;
	}

	return true;
}

bool Save(string fn)
{
	ofstream out(fn.c_str());

	//plot
	SaveCells(out, &plotIndex, 1);

	//nView
	SaveCells(out, &scale[0], 3);
	SaveCells(out, &rotate[0], 3);
	SaveCells(out, &lookat[0], 9);
	SaveCells(out, &perspective[0], 4);

	//nAxis
	SaveCells(out, &max[0], 3);
	SaveCells(out, &min[0], 3);
	SaveCells(out, &inc[0], 3);
	SaveCells(out, &loc[0], 3);
	SaveCells(out, &tinc[0], 3);
	SaveCells(out, &tick[0], 3);

	//nZoom
	SaveCells(out, &zoom, 1);
	SaveCells(out, &center[0], 3);

	//nColor
	SaveCells(out, &zmin, 1);
	SaveCells(out, &zmax, 1);

	out << zcCell.size() << endl;
	for(int i=0; i<zcCell.size(); i++)
		for(int j=0; j<4; j++)
			SaveCells(out, &(zcCell[i][j]), 1);


	//nCamera
	SaveCells(out, &clattitude, 1);
	SaveCells(out, &clongitude, 1);
	SaveCells(out, &cradius, 1);

	//nFont
	SaveCells(out, &fXColor[0], 3);
	SaveCells(out, &fYColor[0], 3);
	SaveCells(out, &fZColor[0], 3);
	SaveCells(out, &fSize[0], 3);
	
	out.close();

	return true;
};

bool Load(string fn)
{
	ifstream in(fn.c_str());

	//plot
	LoadCells(in, &plotIndex, 1);

	//nView
	LoadCells(in, &scale[0], 3);
	LoadCells(in, &rotate[0], 3);
	LoadCells(in, &lookat[0], 9);
	LoadCells(in, &perspective[0], 4);

	//nAxis
	LoadCells(in, &max[0], 3);
	LoadCells(in, &min[0], 3);
	LoadCells(in, &inc[0], 3);
	LoadCells(in, &loc[0], 3);
	LoadCells(in, &tinc[0], 3);
	LoadCells(in, &tick[0], 3);

	//nZoom
	LoadCells(in, &zoom, 1);
	LoadCells(in, &center[0], 3);

	//nColor
	LoadCells(in, &zmin, 1);
	LoadCells(in, &zmax, 1);

	int zcCellSize;
	in >> zcCellSize;
	zcCell.erase(zcCell.begin(), zcCell.end());
	zcCell.resize(zcCellSize);

	//cs.erase(cs.begin(), cs.end());
	cs.resize(zcCellSize);

	for(int i=0; i<zcCell.size(); i++)
	{
		zcCell[i].resize(4);
		for(int j=0; j<4; j++)
			LoadCells(in, &(zcCell[i][j]), 1);

		cs[i] = (cColorSet(zcCell[i][0].value,
								cColor(	zcCell[i][1].value,
										zcCell[i][2].value,
										zcCell[i][3].value)));
				
	}

	//nCamera
	LoadCells(in, &clattitude, 1);
	LoadCells(in, &clongitude, 1);
	LoadCells(in, &cradius, 1);

	//nFont
	LoadCells(in, &fXColor[0], 3);
	LoadCells(in, &fYColor[0], 3);
	LoadCells(in, &fZColor[0], 3);
	LoadCells(in, &fSize[0], 3);

	in.close();

	UpdateColor();

	RedisplayAll();

	return true;
}