#include "cWindowPlot.h"
#include "cWindow.h"
#include "cWindowMain.h"
#include "cPlotData.h"
#include "cPlot.h"
#include <cmath>
#if defined(_WIN32)
	#include "cFont.h"
#endif

using namespace std;

cPlot plot;
int old_y;
int old_x;
cell plotIndex = { 1, gapLS+gapNUM*0, gapTOP*1, 0.0, 0.0, 0.0, 0.05, " "/*"Data Set Index."*/, "%.0f" };

cFloat fpeak(cFloat x, cFloat y)
{
	return 1.0/(1.0+x*x*x*x+y*y*y*y*y*y);
}

cFloat f2peak(cFloat x, cFloat y)
{
	x *= 5.0;
	y *= 5.0;

	cFloat sX = -3.0;
	cFloat sY = 3.0;

	cFloat r1 = fpeak(pow(x-sX, 1.0), pow(y, 1.0));
	cFloat r2 = fpeak((x+sX), (y+sY));
	
	return (r1*2.0 + r2);

}

cFloat fwave(cFloat x, cFloat y)
{
	const double cut = 4.0;
	// x %= 6, y %= 6
	while (fabs(x) > cut) x -= (fabs(x))/x*cut;
	while (fabs(y) > cut) y -= (fabs(y))/y*cut;

	if (x < 0) x += cut;
	if (y < 0) y += cut;

	if (fabs(x) > cut/2.0) return fwave(cut-x, y);
	if (fabs(y) > cut/2.0) return fwave(x, cut-y);

	return (1.0 + x*y/6.0)/(1.0 + pow((sin(x) + cos(y)), 2.0));
}

cFloat fcliff(cFloat x, cFloat y)
{
	return 1.0/(1.0 + pow((sin(x) + cos(y)) + abs(y), 2.0));
}

cFloat fcircle(cFloat x, cFloat y)
{
	return 1.0/(1.0+pow(cos((x*x+y*y)), 2));
}

cColor fspectrum(double z)
{
	double maxZ = 1.2;
	double minZ = 0;

	double r = (z - minZ)/(maxZ-minZ);

	
	typedef pair<double, cColor> cColorSet;

	vector<cColorSet> cs;
	cs.push_back(cColorSet(0.0, cColor(0.25, 0.0, 1.0))); //dark blue
	cs.push_back(cColorSet(0.35, cColor(0.0, 0.0, 1.0))); //blue
	cs.push_back(cColorSet(0.50, cColor(0.4, 1.0, 0.4))); //green
	cs.push_back(cColorSet(0.65, cColor(1.0, 1.0, 0.0))); //yellow
	cs.push_back(cColorSet(0.90, cColor(1.0, 0.0, 0.0))); //red
	cs.push_back(cColorSet(1.0, cColor(0.0, 0.0, 0.0))); //black

	const int numSet = cs.size();

	cColorSet max, min;

	for(int s=0; s<numSet-1; s++)
	{
		if (r > cs[s].first && r <= cs[s+1].first)
		{
			min = cs[s];
			max = cs[s+1];
			break;
		}
	}

	if (r <= cs[0].first)
	{
		min = cs[0];
		max = cs[1];
	}

	if (r >= cs[numSet-1].first)
	{
		min = cs[numSet-2];
		max = cs[numSet-1];
	}

	double r2 = (r - min.first) / (max.first - min.first);

	return (max.second - min.second) * r2 + min.second;
}


cColor flin(double z)
{
	//return fspectrum(z);

	double maxZ = 1;
	double minZ = 0;

	double r = (z - minZ)/(maxZ-minZ);

	cColor maxColor(0.1, 0.2, 0.3);
	cColor minColor(0.2, 0.9, 1.0);

	return (maxColor - minColor) * r + minColor;
};

void initPlot()
{
	
	// add data set
	/*
	cDataSet* ds0 = new cDataSet(fpeak, flin);
	cDataSet* ds1 = new cDataSet(fcircle, flin);
	cDataSet* ds2 = new cDataSet(fwave, flin);
	cDataSet* ds3 = new cDataSet(fcliff, flin);

	cDataSet* ds02 = new cDataSet(fpeak, fspectrum);
	cDataSet* ds12 = new cDataSet(fcircle, fspectrum);
	cDataSet* ds22 = new cDataSet(fwave, fspectrum);
	cDataSet* ds32 = new cDataSet(fcliff, fspectrum);
	*/

	cDataSet* ds03 = new cDataSet(fcircle, fcolor);
	cDataSet* ds13 = new cDataSet(fpeak, fcolor);
	cDataSet* ds23 = new cDataSet(fwave, fcolor);
	cDataSet* ds33 = new cDataSet(fcliff, fcolor);

	cDataSet* dgar = new cDataSet(f2peak, fcolor);
	
	plot.AddData(dgar);
	//plot.AddData(ds03);

	plot.AddData(ds13);
	plot.AddData(ds23);
	plot.AddData(ds33);

	/*
	plot.AddData(ds0);
	plot.AddData(ds1);
	plot.AddData(ds2);
	plot.AddData(ds3);
	
	plot.AddData(ds02);
	plot.AddData(ds12);
	plot.AddData(ds22);
	plot.AddData(ds32);
	*/

	glutDisplayFunc(displayPlot);
	glutReshapeFunc(reshapePlot);
    glutMotionFunc(motionPlot);
    glutMouseFunc(mousePlot);

	glutCreateMenu(menuPlot);
	glutAddMenuEntry("Toggle graph", 'g');
	glutAddMenuEntry("Toggle axis", 'a');
	glutAddMenuEntry("Toggle arrow", 'r');
	glutAddMenuEntry("Toggle box", 'b');
	glutAddMenuEntry("Toggle scale", 'c');
	glutAddMenuEntry("", ' ');
	glutAddMenuEntry("Toggle point", 'p');
	glutAddMenuEntry("Toggle frame", 'f');
	glutAddMenuEntry("Toggle surface", 's');
    glutAttachMenu(GLUT_RIGHT_BUTTON);

	//building font
	BuildFont(0.0, fFill.IsOn());

}

void displayPlot()
{
    glClearColor(0.2, 0.2, 0.2, 0.4);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
/*
	//special effect
	glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	double temp = perspective[3].value;
	perspective[3].value = 10.0; //far clip plane
 	plot.Render();
	perspective[3].value = temp;
	
	glDisable(GL_BLEND);
*/
	//actual display start here
 	plot.Render();

	glPushMatrix();
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluOrtho2D(0, pWIDTH, pHEIGHT, 0);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

	    setfont("helvetica", 18);

		glColor3ub(255, 255, 255);
		drawstr(gapLEFT, plotIndex.y, "Data # :");
		drawstr(gapLEFT, plotIndex.y + gapTOP, plot.GetData(plotIndex.value)->GetCaption().c_str());

		cell_draw(&plotIndex);
	} glPopMatrix();

    glutSwapBuffers();
}

void reshapePlot(int width, int height)
{
    glViewport(0, 0, width, height);
    
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    
	glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glClearColor(0.2, 0.2, 0.2, 0.4);
}

int sbut;

void mousePlot(int button, int state, int x, int y)
{
    selection = 0;
    
    if (state == GLUT_DOWN && button == GLUT_LEFT_BUTTON)
	{
		selection += cell_hit(&plotIndex, x, y);
	}

	old_y = y;
	old_x = x;

	sbut = button;


	RedisplayAll();
}

//TODO:  change this later, now I hack in the auto load plot
int pIndex;

void motionPlot(int x, int y)
{
	cell_update(&plotIndex, old_y-y);

	//if change, autoload
	if (int(plotIndex.value + 0.5) != pIndex)
	{
		menuMain('l');
	}

	//if nothing is selected, then it's in changing camera mode
	if (selection == 0 && sbut == GLUT_LEFT_BUTTON)
	{
		cell_update(&clattitude, y-old_y, true);
		cell_update(&clongitude, x-old_x, true);
	}

	old_x = x;
	old_y = y;

	RedisplayAll();

	//hack start
	pIndex = int(plotIndex.value + 0.5);
}

void menuPlot(int value)
{
	switch (value)
	{
	case 'a': plot.ToggleAxis();		break;
	case 'b': plot.ToggleBox();			break;
	case 'g': plot.ToggleGraph();		break;
	case 'p': plot.TogglePoint();		break;
	case 'f': plot.ToggleWireframe();	break;
	case 's': plot.ToggleSurface();		break;
	case 'c': plot.ToggleScale();		break;
	case 'r': plot.ToggleArrow();		break;
	}

	RedisplayAll();
}
