#include "cWindowWorld.h"
#include "cPlot.h"
#include "cWindow.h"
void
identity(GLdouble m[16])
{
    m[0+4*0] = 1; m[0+4*1] = 0; m[0+4*2] = 0; m[0+4*3] = 0;
    m[1+4*0] = 0; m[1+4*1] = 1; m[1+4*2] = 0; m[1+4*3] = 0;
    m[2+4*0] = 0; m[2+4*1] = 0; m[2+4*2] = 1; m[2+4*3] = 0;
    m[3+4*0] = 0; m[3+4*1] = 0; m[3+4*2] = 0; m[3+4*3] = 1;
}

GLboolean
invert(GLdouble src[16], GLdouble inverse[16])
{
    double t;
    int i, j, k, swap;
    GLdouble tmp[4][4];
    
    identity(inverse);
    
    for (i = 0; i < 4; i++) {
        for (j = 0; j < 4; j++) {
            tmp[i][j] = src[i*4+j];
        }
    }
    
    for (i = 0; i < 4; i++) {
        /* look for largest element in column. */
        swap = i;
        for (j = i + 1; j < 4; j++) {
            if (fabs(tmp[j][i]) > fabs(tmp[i][i])) {
                swap = j;
            }
        }
        
        if (swap != i) {
            /* swap rows. */
            for (k = 0; k < 4; k++) {
                t = tmp[i][k];
                tmp[i][k] = tmp[swap][k];
                tmp[swap][k] = t;
                
                t = inverse[i*4+k];
                inverse[i*4+k] = inverse[swap*4+k];
                inverse[swap*4+k] = t;
            }
        }
        
        if (tmp[i][i] == 0) {
        /* no non-zero pivot.  the matrix is singular, which
           shouldn't happen.  This means the user gave us a bad
            matrix. */
            return GL_FALSE;
        }
        
        t = tmp[i][i];
        for (k = 0; k < 4; k++) {
            tmp[i][k] /= t;
            inverse[i*4+k] /= t;
        }
        for (j = 0; j < 4; j++) {
            if (j != i) {
                t = tmp[j][i];
                for (k = 0; k < 4; k++) {
                    tmp[j][k] -= tmp[i][k]*t;
                    inverse[j*4+k] -= inverse[i*4+k]*t;
                }
            }
        }
    }
    return GL_TRUE;
}

float
normalize(float* v)
{
    float length;
    
    length = sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
    v[0] /= length;
    v[1] /= length;
    v[2] /= length;
    
    return length;
}


void displayWorld()
{
    glClearColor(0.0f, 0.0f, 0.0f, 0.0);
	GLdouble projection[16], modelview[16], inverse[16], invModel[16];

	GLdouble *temp1 = plot.GetModel();
	GLdouble *temp2 = plot.GetProj();
	for(int t=0; t<16; t++)
	{
		modelview[t] = temp1[t];
		projection[t] = temp2[t];
	}
	bool world_draw = true;

    GLfloat light_pos[] = { 0.0, 0.0, 1.0, 0.0 };
    double length;
    float l[3];
    
    l[0] = lookat[3].value - lookat[0].value; 
    l[1] = lookat[4].value - lookat[1].value; 
    l[2] = lookat[5].value - lookat[2].value;
    length = normalize(l);
    
    invert(modelview, inverse);
    invert(modelview, invModel);
    
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//draw the viewing frustum
    glPushMatrix();
	{
		invert(projection, inverse);
		glMultMatrixd(inverse);
    
		glColor3f(0.2, 0.2, 0.2);
		glBegin(GL_QUADS);
		glVertex3i(1, 1, 1);
		glVertex3i(-1, 1, 1);
		glVertex3i(-1, -1, 1);
		glVertex3i(1, -1, 1);
		glEnd();
    
		glColor3ub(128, 196, 128);
		glBegin(GL_LINES);
		glVertex3i(1, 1, -1);
		glVertex3i(1, 1, 1);
		glVertex3i(-1, 1, -1);
		glVertex3i(-1, 1, 1);
		glVertex3i(-1, -1, -1);
		glVertex3i(-1, -1, 1);
		glVertex3i(1, -1, -1);
		glVertex3i(1, -1, 1);
		glEnd();
    
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glColor4f(0.2, 0.2, 0.4, 0.5);
		glBegin(GL_QUADS);
		glVertex3i(1, 1, -1);
		glVertex3i(-1, 1, -1);
		glVertex3i(-1, -1, -1);
		glVertex3i(1, -1, -1);
		glEnd();
		glDisable(GL_BLEND);
	} glPopMatrix();
    
	//draw the box
	glPushMatrix();
	{
		glMultMatrixd(invModel);

		glColor3ub(255, 255, 255);

		//dotted line for the part under surface
		glDisable(GL_DEPTH_TEST);
		glEnable(GL_LINE_STIPPLE);

		int stipple = 0x3333;
		glLineStipple(1, (GLushort)stipple);
		plot.RenderBox();

		//solid line for visible part
		glDisable(GL_LINE_STIPPLE);
		glEnable(GL_DEPTH_TEST);
		plot.RenderBox();

	} glPopMatrix();
	
	glutSwapBuffers();


};

void reshapeWorld(int width, int height)
{
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0, (GLfloat)width/height, 1.0, 256.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(0.0, 0.0, -5.0);

    glRotatef(-45.0, 0.0, 1.0, 0.0);
	glScalef(0.25, 0.25, 0.25);
	//glRotatef(-90.0, 1.0, 0.0, 0.0);

    glClearColor(0.0, 0.0, 0.0, 0.0);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHT0);
}
